@@ -13,12 +13,86 @@ On every RapberryPi that you want to use you must have pigpiod daemon running!
 
 You can download pigpiod from here L<http://abyz.co.uk/rpi/pigpio/download.html>
 
+=head2 SYNOPSYS
+
+Blink a led connecyed to GPIO17 on the RasPi connected to the network with ip address 192.168.1.10
+
+    use RPi::PIGPIO ':all';
+
+    my $pi = RPi::PIGPIO->connect('192.168.1.10');
+
+    $pi->set_mode(17, PI_OUTPUT);
+
+    $pi->write(17, HI);
+
+    sleep 3;
+
+    $pi->write(17, LOW);
+
+Easier mode to controll leds / switches :
+
+    use RPi::PIGPIO;
+    use RPi::PIGPIO::Device::LED;
+
+    my $pi = RPi::PIGPIO->connect('192.168.1.10');
+
+    my $led = RPi::PIGPIO::Device::LED->new($pi,17);
+
+    $led->on;
+
+    sleep 3;
+
+    $led->off;
+
+Same with a switch (relay):
+
+    use RPi::PIGPIO;
+    use RPi::PIGPIO::Device::Switch;
+
+    my $pi = RPi::PIGPIO->connect('192.168.1.10');
+
+    my $switch = RPi::PIGPIO::Device::Switch->new($pi,4);
+
+    $switch->on;
+
+    sleep 3;
+
+    $switch->off;
+
+Read the temperature and humidity from a DHT22 sensor connected to GPIO4
+
+    use RPi::PIGPIO;
+    use RPi::PIGPIO::Device::DHT22;
+
+    my $dht22 = RPi::PIGPIO::Device::DHT22->new($pi,4);
+
+    $dht22->trigger(); #trigger a read
+
+    print "Temperature : ".$dht22->temperature."\n";
+    print "Humidity : ".$dht22->humidity."\n";
+
+=head1 SUPPORTED DEVICES
+
+Note: you can write your own code using methods implemeted here to controll your own device
+
+This is just a list of devices for which we already implemented some functionalities to make your life easier
+
+=over 4
+
+=item * DHT22 sensor - use L<RPi::PIGPIO::Device::DHT22>
+
+=item * LED - use L<RPi::PIGPIO::Device::LED>
+
+=item * generic switch / relay - use L<RPi::PIGPIO::Device::LED>
+
+=back
+
 =cut
 
 use strict;
 use warnings;
 
-our $VERSION     = '0.006';
+our $VERSION     = '0.007';
 
 use Exporter 5.57 'import';
 
@@ -183,6 +257,10 @@ use constant {
     PI_CMD_FS    => 108,
     PI_CMD_FL    => 109,
     PI_CMD_SHELL => 110,
+    
+    PI_CMD_BSPIC => 112, # bbSPIClose
+    PI_CMD_BSPIO => 134, # bbSPIOpen
+    PI_CMD_BSPIX => 193, # bbSPIXfer
 };
 
 
@@ -242,6 +320,18 @@ sub connect {
 }
 
 
+=head2 connected
+
+Returns true is we have an established connection with the remote pigpiod daemon
+
+=cut
+sub connected {
+    my $self = shift;
+    
+    return $self->{sock} && $self->{sock}->connected();
+}
+
+
 =head2 disconnect
 
 Disconnect from the gpiod daemon.
@@ -469,6 +559,201 @@ sub gpio_trigger {
     $self->send_command_ext(PI_CMD_TRIG, $gpio, $length, [ $level ]);
 }
 
+
+=head2 spi_open
+
+Comunication is done via harware SPI so MAKE SURE YOU ENABLED SPI on your RPi (use raspi-config command and go to "Advanced")
+
+Returns a handle for the SPI device on channel.  Data will be
+transferred at baud bits per second.  The flags may be used to
+modify the default behaviour of 4-wire operation, mode 0,
+active low chip select.
+
+An auxiliary SPI device is available on all models but the
+A and B and may be selected by setting the A bit in the
+flags. The auxiliary device has 3 chip selects and a
+selectable word size in bits.
+
+    spi_channel:= 0-1 (0-2 for the auxiliary SPI device).
+           baud:= 32K-125M (values above 30M are unlikely to work).
+      spi_flags:= see below.
+
+
+spi_flags consists of the least significant 22 bits.
+
+    21 20 19 18 17 16 15 14 13 12 11 10  9  8  7  6  5  4  3  2  1  0
+    b  b  b  b  b  b  R  T  n  n  n  n  W  A u2 u1 u0 p2 p1 p0  m  m
+
+mm defines the SPI mode.
+
+WARNING: modes 1 and 3 do not appear to work on
+the auxiliary device.
+
+    Mode POL PHA
+    0    0   0
+    1    0   1
+    2    1   0
+    3    1   1
+
+px is 0 if CEx is active low (default) and 1 for active high.
+
+ux is 0 if the CEx GPIO is reserved for SPI (default)
+and 1 otherwise.
+
+A is 0 for the standard SPI device, 1 for the auxiliary SPI.
+
+W is 0 if the device is not 3-wire, 1 if the device is 3-wire.
+Standard SPI device only.
+
+nnnn defines the number of bytes (0-15) to write before
+switching the MOSI line to MISO to read data.  This field
+is ignored if W is not set.  Standard SPI device only.
+
+T is 1 if the least significant bit is transmitted on MOSI
+first, the default (0) shifts the most significant bit out
+first.  Auxiliary SPI device only.
+
+R is 1 if the least significant bit is received on MISO
+first, the default (0) receives the most significant bit
+first.  Auxiliary SPI device only.
+
+bbbbbb defines the word size in bits (0-32).  The default (0)
+sets 8 bits per word.  Auxiliary SPI device only.
+
+The C<spi_read>, C<spi_write>, and C<spi_xfer> functions
+transfer data packed into 1, 2, or 4 bytes according to
+the word size in bits.
+
+For bits 1-8 there will be one byte per character. 
+For bits 9-16 there will be two bytes per character. 
+For bits 17-32 there will be four bytes per character.
+
+E.g. 32 12-bit words will be transferred in 64 bytes.
+
+The other bits in flags should be set to zero.
+
+
+Example: open SPI device on channel 1 in mode 3 at 50k bits per second
+
+    my $spi_handle = $pi.spi_open(1, 50_000, 3);
+
+=cut
+sub spi_open {
+    my ($self, $spi_channel, $baud, $spi_flags) = @_;
+    
+    $spi_flags //= 0;
+    
+    return $self->send_command_ext(PI_CMD_SPIO, $spi_channel, $baud, [ $spi_flags ]);
+}
+
+
+=head2 spi_close
+
+Closes an SPI channel
+
+Usage :
+
+    my $spi = $pi->spi_open(0,32_000);
+    ... 
+    $pi->spi_close($spi);
+
+=cut
+sub spi_close {
+    my ($self, $handle) = @_;
+    
+    return $self->send_command(PI_CMD_SPIC, $handle, 0);
+}
+   
+
+=head2 spi_read
+
+Arguments (in this order):
+
+handle:= >=0 (as returned by a prior call to C<spi_open>).
+count:= >0, the number of bytes to read.
+
+The returned value is a bytearray containing the bytes.
+
+Usage: 
+
+    my $data = $pi->spi_read(12);
+
+=cut
+sub spi_read {
+    my ($self, $handle, $count) = @_;
+    
+    $self->send_command(PI_CMD_SPIR, $handle, $count);
+    
+    my $response;
+    
+    $self->{sock}->recv($response,3);
+    
+    return $response;
+}
+
+
+=head2 spi_write
+
+Writes the data bytes to the SPI device associated with handle.
+
+Arguments (in this order):
+
+handle:= >=0 (as returned by a prior call to C<spi_open>).
+data:= the bytes to write.
+
+Examples : 
+
+    $pi->spi_write(1, [2, 192, 128]);      # write 3 bytes to device 1
+
+    $pi->spi_write(0, 'defgh');            # write 5 bytes to device 0
+
+=cut
+sub spi_write {
+    my ($self, $handle, $data) = @_;
+    
+    if (! ref($data) ) {
+        $data = [ map {ord} split("",$data) ];
+    }
+    
+    return $self->send_command_ext(PI_CMD_SPIW, $handle, 0, $data);
+}
+
+=head2 spi_xfer
+
+Writes the data bytes to the SPI device associated with handle,
+returning the data bytes read from the device.
+
+Arguments (in this order):
+
+handle:= >=0 (as returned by a prior call to C<spi_open>).
+data:= the bytes to write.
+
+The returned value is a bytearray containing the bytes.
+
+Examples :
+
+    my $rx_data = $pi->spi_xfer(h, [1, 128, 0]);
+
+=cut
+sub spi_xfer {
+    my ($self, $handle, $data) = @_;
+    
+    if (! ref($data) ) {
+        $data = [ map {ord} split("",$data) ];
+    }
+    
+    my $bytes = $self->send_command_ext(PI_CMD_SPIX, $handle, 0, $data);
+    
+    my $response;
+    
+    $self->{sock}->recv($response,$bytes);
+    
+    return $response;
+}
+
+
+################################################################################################################################
+
 =head1 PRIVATE METHODS
 
 =cut
@@ -581,7 +866,7 @@ sub send_command_ext {
 sub prepare_for_exit {
     my $self = shift;
     
-    $self->{sock}->close();
+    $self->{sock}->close() if $self->{sock};
 }
 
 sub DESTROY {
